
/*  string utilities for HITRAN records
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "hdefs.h"


/*  copy exactly n characters from s2 to s1 and
 *  add a null at the end of s1, at position n+1.
 */
void scp(char *a, char *b, int n) {
  int i=0;
  char *p, *q;
  for (p=a, q=b; i < n; i++) *p++ = *q++;
  *p = 0;
}


/* copy a raw record to a null-terminated string record
 */
void raw2str (struct HRAW *hraw, struct HSTR *hstr) {

  scp(hstr->IGAS,   hraw->IGAS,    2);
  scp(hstr->ISO,    hraw->ISO,     1);
  scp(hstr->WNUM,   hraw->WNUM,    12);
  scp(hstr->STREN,  hraw->STREN,   10);
  scp(hstr->TPROB,  hraw->TPROB,   10);
  scp(hstr->ABROAD, hraw->ABROAD,  5);
  scp(hstr->SBROAD, hraw->SBROAD,  5);
  scp(hstr->ELS,    hraw->ELS,     10);
  scp(hstr->ABCOEF, hraw->ABCOEF,  4);
  scp(hstr->TSP,    hraw->TSP,     8);
  scp(hstr->IUSGQ,  hraw->IUSGQ,   3);
  scp(hstr->ILSGQ,  hraw->ILSGQ,   3);
  scp(hstr->USLQ,   hraw->USLQ,    9);
  scp(hstr->BSLQ,   hraw->BSLQ,    9);
  scp(hstr->AI,     hraw->AI,      3);
  scp(hstr->REF,    hraw->REF,     6);
} 


/* copy a null-terminated string record back to a raw record
 */
void str2raw (struct HSTR *hstr, struct HRAW *hraw) {

  scp(hraw->IGAS,   hstr->IGAS,    2);
  scp(hraw->ISO,    hstr->ISO,     1);
  scp(hraw->WNUM,   hstr->WNUM,    12);
  scp(hraw->STREN,  hstr->STREN,   10);
  scp(hraw->TPROB,  hstr->TPROB,   10);
  scp(hraw->ABROAD, hstr->ABROAD,  5);
  scp(hraw->SBROAD, hstr->SBROAD,  5);
  scp(hraw->ELS,    hstr->ELS,     10);
  scp(hraw->ABCOEF, hstr->ABCOEF,  4);
  scp(hraw->TSP,    hstr->TSP,     8);
  scp(hraw->IUSGQ,  hstr->IUSGQ,   3);
  scp(hraw->ILSGQ,  hstr->ILSGQ,   3);
  scp(hraw->USLQ,   hstr->USLQ,    9);
  scp(hraw->BSLQ,   hstr->BSLQ,    9);
  scp(hraw->AI,     hstr->AI,      3);
  scp(hraw->REF,    hstr->REF,     6);
  hraw->NL[0] = '\n';
} 


/* copy a null-terminated string record to a value record
 */
void str2val (struct HSTR *hstr, struct HVAL *hval) {

  hval->IGAS    =  atoi(hstr->IGAS);
  hval->ISO     =  atoi(hstr->ISO);
  hval->WNUM    =  atof(hstr->WNUM);
  hval->STREN   =  atof(hstr->STREN);
  hval->TPROB   =  atof(hstr->TPROB);
  hval->ABROAD  =  atof(hstr->ABROAD);
  hval->SBROAD  =  atof(hstr->SBROAD);
  hval->ELS     =  atof(hstr->ELS);
  hval->ABCOEF  =  atof(hstr->ABCOEF);
  hval->TSP     =  atof(hstr->TSP);
  hval->IUSGQ   =  atoi(hstr->IUSGQ);
  hval->ILSGQ   =  atoi(hstr->ILSGQ);
  scp(hval->USLQ,  hstr->USLQ,  9);
  scp(hval->BSLQ,  hstr->BSLQ,  9);
  scp(hval->AI,    hstr->AI,    3);
  scp(hval->REF,   hstr->REF,   6);
}


/* copy a value record to a null-terminated string record
 */
static void verr(char *s) {
/*     fprintf(stderr, "val2str(): bad %s length\n", s); */
/*     exit(127); */
}

void val2str (struct HVAL *hval, struct HSTR *hstr) {

char b[8];

  if (sprintf(hstr->IGAS,   "%2d",    hval->IGAS)   != 2)  verr("IGAS");
  if (sprintf(hstr->ISO,    "%1d",    hval->ISO)    != 1)  verr("ISO");
  if (sprintf(hstr->WNUM,   "%12.6f", hval->WNUM)   != 12) verr("WNUM");
  if (sprintf(hstr->STREN,  "%10.3e", hval->STREN)  != 10) verr("STREN");
  if (sprintf(hstr->TPROB,  "%10.3e", hval->TPROB)  != 10) verr("TPROB");

  /* a hack to strip off the leading zero for a fortran-style F5.4 */

  if (sprintf(b, "%6.4f",  hval->ABROAD) != 6)  verr("ABROAD");
  scp(hstr->ABROAD, b+1, 5);

  if (sprintf(b, "%6.4f",  hval->SBROAD) != 6)  verr("SBROAD");
  scp(hstr->SBROAD, b+1, 5);

  if (sprintf(hstr->ELS,    "%10.4f", hval->ELS)    != 10) verr("ELS");
  if (sprintf(hstr->ABCOEF, "%4.2f",  hval->ABCOEF) != 4)  verr("ABCOEF");
  if (sprintf(hstr->TSP,    "%8.6f",  hval->TSP)    != 8)  verr("TSP");
  if (sprintf(hstr->IUSGQ,  "%3d",    hval->IUSGQ)  != 3)  verr("IUSGQ");
  if (sprintf(hstr->ILSGQ,  "%3d",    hval->ILSGQ)  != 3)  verr("ILSGQ");
  if (sprintf(hstr->USLQ,   "%9s",    hval->USLQ)   != 9)  verr("USLQ");
  if (sprintf(hstr->BSLQ,   "%9s",    hval->BSLQ)   != 9)  verr("BSLQ");
  if (sprintf(hstr->AI,     "%3s",    hval->AI)     != 3)  verr("AI");
  if (sprintf(hstr->REF,    "%6s",    hval->REF)    != 6)  verr("REF");
}


/*  write a "value record" in readable format
 */
void write_hval (struct HVAL *hval, FILE *fid) {

  fprintf(fid, 
	  "%2d %1d %12.6f %10.3e %10.3e %6.4f %6.4f %10.4f %4.2f %8.6f %3d %3d %9s %9s %3s %6s\n",
	 hval->IGAS,
	 hval->ISO,
	 hval->WNUM,
	 hval->STREN,
	 hval->TPROB,
	 hval->ABROAD,
	 hval->SBROAD,
	 hval->ELS,
	 hval->ABCOEF,
	 hval->TSP,
	 hval->IUSGQ,
	 hval->ILSGQ,
	 hval->USLQ,
	 hval->BSLQ,
	 hval->AI,
	 hval->REF
	 );
}


/* write a raw record
 */
void write_hraw (struct HRAW *hraw, FILE *fid) {

  fwrite(hraw, sizeof(struct HRAW), 1, fid);
}


/* write a string-format record 
 * (mainly for testing purposes...)
 */
void write_hstr (struct HSTR *hstr, FILE *fid) {
  int i;
  char c;
  for (i=0; i<sizeof(struct HSTR); i++) {
    c = ((char *) hstr)[i];
    if (c == 0) 
      putc(' ', fid);
    else
      putc(c, fid); 
  }
  putc('\n', fid);
}

